;**************************************************************************
;*
;* Title:		fastAVRlink
;* Version:		1.3
;* Last updated:	00.10.01
;* Target:		AT90S2313
;*
;* Support E-mail:	rlievin@mail.com
;*
;* Code Size		:?? words
;* Low Register Usage	:?
;* High Register Usage	:?
;* Interrupt Usage	:2 interrupts (RXC & DRE)
;*
;* DESCRIPTION
;*
;* This program clones the Grey TI Graph Link cable of Texas Intruments.
;* Version 1.1: UART used with 2 buffers and up to 38400 bauds.
;* Version 1.2: UART used with 2 buffers and up to 57600 bauds with
;* hardware flow control (only CTS).
;* Version 1.3: EEPROM used for storing some parameters (external setup)
;* Version 1.4: full hardware flow control (CTS and RTS)
;*
;**************************************************************************

;.include "2313def.inc"


;**************************************************************************
;*
;* eep_read - Subroutine for reading a value in EEPROM
;*
;*
;* DESCRIPTION
;* This routine reads a byte from the internal EEPROM. 
;* The address is passed by the 'adr' register and the read data is put
;* in the 'data' register.
;*
;*
;* Total number of words	: ?
;* Total number of cycles	: ?? (including the RET instructions)
;* Low register usage		: 2
;* High register usage		: 2
;* 
;**************************************************************************

eep_read:
	out	EEAR,adr		;Set the EEPROM's address
	sbi	EECR,EERE		;Send the Read strobe
	in	data,EEDR		;Put the data in the transmit register
	ret

;**************************************************************************
;*
;*	EEPROM TABLE - Configurable parameters
;*
;* Rem: all strings are NULL terminated !
;*
;**************************************************************************
.eseg
; Parameters (@0h, 8 bytes)
params:
ls_o:
	.db	N19200	;link cable speed, 19200 by default @11.0592MHz
ibt_o:
	.db	4	;inter-bit tempo
bs_o:
	.db	32	;buffer size
bp_o:
	.db	1	;pass-through enabled
hfc_o:
	.db	1	;hardware flow control disabled
wdt_o:
	.db	1	;watchdog enabled
	.db	0
	.db	0
;Version number (@8h, 8 bytes)
version:
	.db	'V'	;"V1.60"
	.db	'1'	;
	.db	'.'	;
	.db	'6'	;
	.db	'0'	;
	.db	0
	.db	0
	.db	0
copyright:
;Copyright (@10h, 32 bytes)
	.db	'C'	;
	.db	'o'
	.db	'p'
	.db	'y'
	.db	'r'
	.db	'i'
	.db	'g'
	.db	'h'
	.db	't'
	.db	' '
	.db	'2'
	.db	'0'
	.db	'0'
	.db	'0'
	.db	' '
	.db	'R'
	.db	'.'
	.db	'L'
	.db	'i'
	.db	'e'
	.db	'v'
	.db	'i'
	.db	'n'
	.db	0
	.db	0
	.db	0
	.db	0
	.db	0
	.db	0
	.db	0
	.db	0
	.db	0
xtal:
; Xtal frequency for the fastAVRlink Setup Utility (@30h, 8 bytes)
	.db	'1'	;11.0592 MHz
	.db	'1'
	.db	'.'
	.db	'0'
	.db	'5'
	.db	'9'
	.db	'2'
	.db	0
	.db	0