;**************************************************************************
;*
;* Title:		Test_232
;* Version:		1.0
;* Last updated:	00.09.12
;* Target:		AT90S2313
;*
;* Support E-mail:	rlievin@mail.com
;*
;* Code Size		:72 words
;* Low Register Usage	:2
;* High Register Usage	:5
;* Interrupt Usage	:nothing
;*
;* DESCRIPTION
;*
;* This application note tests the rs232buf module.
;*
;**************************************************************************

.include "2313def.inc"

;***** WatchDog settings

;.equ	WDP=4	;0.24s at 5.0V (0.75s at 3.0V)
.equ	WDP=5	;0.49s at 5.0V (1.5s at 3.0V)
;.equ	WDP=6	;0.97s at 5.0V


;**************************************************************************
;*
;*	VECTOR TABLE - EXECUTION STARTS HERE
;*
;**************************************************************************
	.cseg

	.org 	$0000
	rjmp	RESET			;Reset handler

	.org 	INT0addr
	rjmp	EXT_INT0		;External interrupt 0 handler

	.org 	INT1addr
	rjmp	EXT_INT1		;External interrupt 0 handler

	.org 	ICP1addr
	rjmp	TIM_CAPT1		;Timer/counter1 capture event handler

	.org	OC1addr
	rjmp	TIM_COMP1		;Timer/counter1 compare match handler

	.org	OVF1addr
	rjmp	TIM_OVF1		;Timer/counter1 overflow handler

	.org	OVF0addr
	rjmp	TIM_OVF0		;Timer/counter0 coverflow handler

	.org	URXCaddr
	rjmp	UART_RXC		;UART, RX complete handler

	.org	UDREaddr
	rjmp	UART_DRE		;UART, Data register empty handler

	.org	UTXCaddr
	rjmp	UART_TXC		;UART, TX complete handler

	.org ACIaddr
	rjmp	ANA_COMP		;Analog comparator handler

; unused vectors: return of interrupt

EXT_INT0:
EXT_INT1:
TIM_CAPT1:
TIM_COMP1:
TIM_OVF1:
TIM_OVF0:
;UART_RXC:
;UART_DRE:
UART_TXC:
ANA_COMP:
	reti

;***** Include subroutines

;.include "rs232buf.asm"	:to use RS232 subroutines (buffer)
;.include "ti_prot.asm"	;to use TI subroutines

;**************************************************************************
;*
;* Title:		Full Duplex Interrupt Driven RS232 with buffers 
;* Version:		1.0 ()
;* Last updated:	00.09.15
;* Target:		AT90S2313
;*
;* Support E-mail:	rlievin@mail.com
;*
;* Code Size		:?? words
;* Low Register Usage	:10
;* High Register Usage	:10
;* Interrupt Usage	:2 UART interrupts
;*
;* DESCRIPTION
;*
;* This application implements a full duplex buffered RS232 communication.
;* It use for this some RAM and the UART as well as some interrupts
;* routines.
;*
;* 
;*
;*
;* *** Initialization
;*
;*  1. Call rs_init
;*  2. Enable global interrupts (with 'sei')
;*
;* *** Receive
;*
;*  1. Wait until RX_EMPTY in 'u_status' becomes clear
;*  2. Call rs_get
;*
;* *** Transmit
;*
;* (0. Initialize the UART by executing uart_init and sei)
;*  1. Wait until TX_FULL in 'u_status' becomes clear
;*  2. Set 'rs_buffer'
;*  3. Call 'rs_put'
;*
;**************************************************************************

;.include "2313def.inc"

;***** BAUD-rate settings at 10 Mhz for the UBRR register

				;BAUD-RATES @10MHz XTAL
.equ	N=64			; 9600
;.equ	N=42			;14400 
;.equ	N=32			;19200
;.equ	N=21			;28800
;.equ	N=15			;38400
;.equ	N=10			;57600

;***** rs232 Global Registers

.def	rs_tmp		=r22	;Temporary storage register (irq routines)
.def	rs_scratch	=r23	;Scratch register (normal routines)
.def	rs_data		=r11	;Serial buffer
.def	rs_status	=r21	;Status buffer

;**** rs232 Buffers: 32 bytes max for each buffer
;* Note: if half-duplex, the pointers can be the same which allow
;* bigger buffer.

.def	rx_rd_ptr	=r13	;receive pointer for reading from buffer
.def	rx_wr_ptr	=r28	;receive pointer for writing to buffer (YL)
.def	rx_counter	=r24	;receive buffer counter
.equ	RX_BUFFER	=$60	;address of receive buffer (RAM)
.equ	RX_BUF_SIZE	=4	;buffer size, must be a power of 2

.def	tx_rd_ptr	=r26	;transmit pointer for reading from buffer (XL)
.def	tx_wr_ptr	=r12	;transmit pointer for writing to buffer
.def	tx_counter	=r25	;transmit buffer counter
.equ	TX_BUFFER	=$80	;address of transmit buffer (RAM)
.equ	TX_BUF_SIZE	=4	;buffer size, must be a power of 2


.equ	BUFFER_UP	=12;	;drop CTS if this limit is reached (2/3)
.equ	BUFFER_DOWN	=4	;raise CTS if this limit is reached (1/3)

;***** Bit positions in the rs232 status-register

.equ	RX_FULL		=5	;receive buffer full
.equ	RX_EMPTY	=4	;receive buffer empty

.equ	TX_FULL		=1	;transmit buffer full
.equ	TX_EMPTY	=0	;transmit buffer empty


;***** Bit positions of the different lines

.equ	TxD=1			;Transmit line 		  (port D)
.equ	RxD=0			;Receive line 		  (port D)
.equ	CTS=6			;Clear To Send line 	  (port B)
.equ	RTS=5			;Request To Send line 	  (port B)
.equ	DTR=7			;Data Terminal Ready line (port B)
; DSR is not connected !


;**************************************************************************
;*
;* rs_init - Subroutine for RS232 initialization
;*
;*
;* DESCRIPTION
;* This routine initializes the UART. It sets the baud-rate register,
;* some flags and sets the buffer pointers.
;* To enable the UART the global interrupt flag must be set (with SEI).
;*
;*
;* Total number of words	: 8
;* Total number of cycles	: 11 (including the RET instructions)
;* Low register usage		: None
;* High register usage		: 2 (u_tmp,u_status)
;* 
;**************************************************************************

rs_init:
	ldi	rs_tmp,N		;init the baud-rate generator
	out	UBRR,rs_tmp
	sbi	UCR,TXEN		;transmitter enable
	sbi	UCR,RXEN		;receiver enable

	ldi	ZH,0			;init the Z pointer
	ldi	ZL,0
	ldi	XH,0
	ldi	XL,TX_BUFFER		;init the tx_rd_ptr
	mov	tx_wr_ptr,XL		;init the tx_wr_ptr
	ldi	YH,0
	ldi	YL,RX_BUFFER		;init the rx_wr_ptr
	mov	rx_rd_ptr,YL		;init the rx_rd_ptr
	mov	rx_counter,ZL		;init counters
	mov	tx_counter,ZL

	ldi	rs_tmp,0b00010001	;init the rs232 status register
	mov	rs_status,rs_tmp

	sbi	UCR,RXCIE		;enable rx interrupt but not tx yet
	ret


;**************************************************************************
;*
;* rs_put - Subroutine for UART transmittal
;*
;*
;* DESCRIPTION
;* This routine place the data to be sent in a buffer. 
;* This data must be located in rs_data. 
;*
;* This routine modify the TX_EMPTY flag.
;*
;*
;* Total number of words	: 13
;* Total number of cycles	: 17 (including RET)
;* Low register usage		: 1 (u_buffer)
;* High register usage		: 4 (u_bit_cnt,u_tmp,u_transmit,u_reload)
;* 
;**************************************************************************

rs_put:
rs_put1:
	cpi	tx_counter,TX_BUF_SIZE	;buffer full ?
	breq	rs_put1			;then wait
	mov	ZL,tx_wr_ptr		;retrieve pointer
	st	Z+,rs_data		;put data in the buffer
	inc	tx_counter		;update counter
	cpi	tx_counter,TX_BUF_SIZE	;if buffer full
	brne	rs_put3
	sbr	rs_status,1<<TX_FULL	;set flag
rs_put3:
	cbr	rs_status,1<<TX_FULL	;else clear
	cpi	ZL,TX_BUFFER+TX_BUF_SIZE	
					;check if pointer is out
	brne	rs_put2
	ldi	ZL,TX_BUFFER		;then wraparound the pointer
rs_put2:
	mov	tx_wr_ptr,ZL		;and update pointer
	sbic	UCR,UDRIE		;return if tx is already launched	
	ret
	sbi	UCR,UDRIE		;triggers the tx interrupt
	ret				;return


;**************************************************************************
;*
;* rs_get - Subroutine for UART receiving
;*
;*
;* DESCRIPTION
;* This routine checks if the receiving buffer is empty and if not, it gets
;* a character from the buffer and places it in rs_data.
;*
;* Checks the R_NEMPTY bit to know there is a char to retrieve.
;*
;*
;* Total number of words	: ??
;* Total number of cycles	: ?? (including RET)
;* Low register usage		: ? ()
;* High register usage		: ? ()
;* 
;**************************************************************************

rs_get:
	cpi	rx_counter,0		;if buffer is empty
	breq	rs_get			;then wait
rs_get1:
	dec	rx_counter		;decrease the counter
	mov	ZL,rx_rd_ptr		;retrieve pointer
	ld	rs_data,Z+		;get data from the buffer
	cpi	ZL,RX_BUFFER+RX_BUF_SIZE	
					;check if pointer is out
	brne	rs_get2
	ldi	ZL,RX_BUFFER		;then wraparound the pointer
rs_get2:
	mov	rx_rd_ptr,ZL		;and update pointer
	ret

;**************************************************************************
;*
;* UART_DRE - UDR Empty Handler Interrupt
;*
;*
;* DESCRIPTION
;* This routine coordinates the transmition and reception of bits. This 
;* routine is automatically executed at a rate equal to the baud-rate. When
;* transmitting, this routine shifts the bits and sends it. When receiving,
;* it samples the bit and shifts it into the buffer.
;*
;* The serial routines uses a status register (u_status): READ-ONLY.
;*		BUSY	This bit indicates whenever the UART is busy
;*		TD	Transmit Data. Set when the UART is transmitting
;*		RDR	Receive Data Ready. Set when new data has arrived
;*			and it is ready for reading.
;*
;* When the RDR flag is set, the (new) data can be read from u_buffer.
;*
;* This routine also sets bits in TIMSK and TIMSK. See ext_int0 description
;*
;* 
;* Total number of words	: 35
;* Total number of cycles	: min. 18, max. 28 - depending on if it is
;*				  receiving or transmitting, what bit is
;*				  pending, etc.
;* Low register usage		: 2 (u_sr,u_buffer)
;* High register usage		: 4 (u_bit_cnt,u_tmp,u_status,u_reload)
;* 
;**************************************************************************

UART_DRE:
	in	rs_tmp,SREG		;Save the Status Register
	push	rs_tmp
	cpi	tx_counter,0		;is buffer empty ?
	brne	udre2			;if not, jump
	cbi	UCR,UDRIE		;disable tx interrupt
	rjmp	udre3	
udre2:
	ld	rs_tmp,X+		;get char from buffer
	out	UDR,rs_tmp		;and transmit in
	dec	tx_counter		;update counter
	brne	udre5			;if buffer empty
	sbr	rs_status,1<<TX_EMPTY	;set flag
udre5:
	cbr	rs_status,1<<TX_EMPTY	;else clear flag
	cpi	XL,TX_BUFFER+TX_BUF_SIZE	
					;check if pointer is out
	brne	udre3
	ldi	ZL,TX_BUFFER		;then wraparound the pointer
udre3:
	pop	rs_tmp			;Restore the Status Register
	out	SREG,rs_tmp
	reti

;**************************************************************************
;*
;* UART_RXC - UDR Empty Handler Interrupt
;*
;*
;* DESCRIPTION
;* This routine coordinates the transmition and reception of bits. This 
;* routine is automatically executed at a rate equal to the baud-rate. When
;* transmitting, this routine shifts the bits and sends it. When receiving,
;* it samples the bit and shifts it into the buffer.
;*
;* The serial routines uses a status register (u_status): READ-ONLY.
;*		BUSY	This bit indicates whenever the UART is busy
;*		TD	Transmit Data. Set when the UART is transmitting
;*		RDR	Receive Data Ready. Set when new data has arrived
;*			and it is ready for reading.
;*
;* When the RDR flag is set, the (new) data can be read from u_buffer.
;*
;* This routine also sets bits in TIMSK and TIMSK. See ext_int0 description
;*
;* 
;* Total number of words	: 35
;* Total number of cycles	: min. 18, max. 28 - depending on if it is
;*				  receiving or transmitting, what bit is
;*				  pending, etc.
;* Low register usage		: 2 (u_sr,u_buffer)
;* High register usage		: 4 (u_bit_cnt,u_tmp,u_status,u_reload)
;* 
;**************************************************************************

UART_RXC:
	in	rs_tmp,SREG		;Save the Status Register
	push	rs_tmp

	in	rs_tmp,UDR		;retrieve char
	st	Y+,rs_tmp		;and put it in the buffer
	inc	rx_counter		;update counter
	cpi	YL,RX_BUFFER+RX_BUF_SIZE	
					;check if pointer is out
	brne	rxc3
	ldi	YL,RX_BUFFER		;then wraparound the pointer
rxc3:
	pop	rs_tmp			;Restore the Status Register
	out	SREG,rs_tmp
	reti


;***** Initialization (common part) 

        .def   	init	=r16
	.def	data	=r17

RESET:  
	ldi	init,low(RAMEND)	;init Stack Pointer	
	out	SPL,init
	ldi	init,0b00000010	 	;set port D bit 1 to output
	out	DDRD,init
	clr	init
	out     PORTB,init		;all outputs pins at 0
        out     DDRB,init		;all pins in input (Hi-Z)
	ldi	init,N			;init the baud rate generator
	out	UBRR,init
	sbi	UCR,TXEN		;transmitter enable
	sbi	UCR,RXEN		;receiver enable
	
;wait:	rcall	check			;wait that red=1 & white=1 (watchdog reset)
;	brtc	wait			;
;	sei				;enable interrupts
;	ldi	scratch,(1<<WDE) + WDP	;load the watchdog prescaler 
;	out	WDTCR,scratch		;and enable watchdog


;**************************************************************************
;*
;*	Test/Example Program
;*
;**************************************************************************

	rcall	rs_init			;init rs232 routines
	sei				;enable interrupts

	ldi	data,'S'
	mov	rs_data,data
	rcall	rs_put
;	ldi	data,'a'
;	mov	rs_data,data
;	rcall	rs_put
;	ldi	data,'l'
;	mov	rs_data,data
;	rcall	rs_put
;	ldi	data,'u'
;	mov	rs_data,data
;	rcall	rs_put
;	ldi	data,'t'
;	mov	rs_data,data
;	rcall	rs_put
;	ldi	data,'!'
;	mov	rs_data,data
;	rcall	rs_put

endloop:
	rcall	rs_get
	rcall	rs_put

	rjmp	endloop

.exit

;**************************************************************************
;*
;*      Test program for TI routines
;*
;**************************************************************************
ti_sub:	
	ldi	ti_reg,0x09
	rcall	ti_put
	ldi	ti_reg,0x68
	rcall	ti_put
	ldi	ti_reg,0x00
	rcall	ti_put
	ldi	ti_reg,0x00
	rcall	ti_put
	rcall	ti_get
	rcall	ti_get
	rcall	ti_get
	rcall	ti_get
jmp:	rjmp	jmp

.exit
