/*  gtktilink - link program for TI calculators
 *  Copyright (C) 1999, 2000  Romain Lievin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Windows code only, no linux code here */
#if defined(__WIN32__)
#include <stdio.h>
#include <windows.h>

//#include "win_port.h"
#include "ioports.h"

#define IO_ASM 0 // low level routines via asm
#define IO_DLL 1 // low level routines via the DLPortIO driver

static int io_access_type = IO_ASM; 

typedef UCHAR (CALLBACK* DLPORTREADPORTUCHAR)  (ULONG);
typedef VOID  (CALLBACK* DLPORTWRITEPORTUCHAR) (ULONG, UCHAR);

DLPORTREADPORTUCHAR DlPortReadPortUchar;
DLPORTWRITEPORTUCHAR DlPortWritePortUchar;

int ioperm (unsigned long from, unsigned long num, int turn_on)
{
	static HINSTANCE hDLL = NULL;		// Handle to DLL
	
	if(hDLL != NULL)
		FreeLibrary(hDLL);
	hDLL = LoadLibrary("DLPORTIO.DLL");
	if (hDLL == NULL)
	{
		printf("DLPortIO library not found. Use defaults I/O routines.\n");
		io_access_type = IO_ASM;
	}
	else
	{
		printf("DLPortIO library loaded.\n");
		io_access_type = IO_DLL;
		DlPortReadPortUchar = (DLPORTREADPORTUCHAR)GetProcAddress(hDLL,
                                           "DlPortReadPortUchar");
		if (!DlPortReadPortUchar)
		{
			// handle the error
			printf("Unable to load DlPortReadPortUchar symbol.\n");
			FreeLibrary(hDLL);       
			return -1;
		}
		DlPortWritePortUchar = (DLPORTWRITEPORTUCHAR)GetProcAddress(hDLL,
                                           "DlPortWritePortUchar");
		if (!DlPortWritePortUchar)
		{
			// handle the error
			printf("Unable to load DlPortWritePortUchar symbol.\n");
			FreeLibrary(hDLL);       
			return -1;
		}
		printf("I/O functions initialized.\n");
	}
	return 0;
}

unsigned char inb (int portid)
{ 
	if(io_access_type == IO_ASM)
		return inb_(portid);
	else
		return DlPortReadPortUchar(portid);
}

void outb (unsigned char value, int portid)
{ 
	if(io_access_type == IO_ASM)
		outb_(value, portid);
	else
		DlPortWritePortUchar(portid, value);
}

#endif
